// PV227, Lesson 13: Parallax Occlusion Mapping
#pragma once

#include "../../Framework/PV227.h"

using namespace std;
using namespace PV227;

//---------------------
//----    SCENE    ----
//---------------------

// Shader programs
ShaderProgram nolit_program;
ShaderProgram object_program;

// Geometries we use in this lecture
Geometry geom_cube;
Geometry geom_sphere;

// A camera that allows us to look at the object from different views
SimpleCamera the_camera;

// Textures
GLuint chesterfield_color_tex;
GLuint chesterfield_normal_tex;
GLuint chesterfield_height_tex;

// Data of our materials
MaterialData_UBO WhiteMaterial_ubo;

// Position of the central object and the light
ModelData_UBO CentralObjectModel_ubo[2];
ModelData_UBO LightModel_ubo;

// Data of all our lights - positions, colors, etc.
PhongLightsData_UBO PhongLights_ubo;

// Data of our camera - view matrix, projection matrix, etc.
CameraData_UBO CameraData_ubo;

// OpenGL query object to get render time of one frame
GLuint RenderTimeQuery;

// Functions that works with scene objects
void reload_shaders();
void init_scene();
void update_scene(int app_time_diff_ms);
void render_scene();

//-------------------
//----    GUI    ----
//-------------------

// Main AntTweakBar object
TwBar *the_gui;

// Variables that are changed with GUI
float render_time_ms;		// Read-only
float light_pos;
float object_distance;
float depth_scale;
int sample_count;
bool render_second_cube;

// Callbacks from the GUI
void TW_CALL reload(void *);

// Functions that works with GUI
void init_gui();

//---------------------------
//----    APPLICATION    ----

// Current window size
int win_width = 640;
int win_height = 480;

// Current time of the application in milliseconds, for animations
int app_time_ms = 0;
int last_glut_time = 0;
